package map_service

import (
	"archive/zip"
	"context"
	"fmt"
	"github.com/cloudwego/hertz/pkg/app"
	"github.com/cloudwego/hertz/pkg/protocol/consts"
	"io"
	"net/http"
	"os"
	"path/filepath"
	"pji_desktop_http/common/config"
	"pji_desktop_http/common/config/c_log"
	"pji_desktop_http/common/entity"
	"pji_desktop_http/common/util"
)

// CheckMapBufConsistency 检查请求id对应的mapBuf文件夹的一致性
// @router /map/checkmapbuf [POST]
func CheckMapBufConsistency(ctx context.Context, c *app.RequestContext) {
	var req []string
	err := c.BindAndValidate(&req)
	if err != nil {
		c.String(consts.StatusBadRequest, err.Error())
		return
	}

	fmt.Println(req)

	if len(req) == 0 { // 请求数据为空
		c.JSON(consts.StatusBadRequest, entity.HttpResult{Status: false, Code: "", Message: "请求数据为空。"})
	} else if len(req) == 1 { // 只有一条数据则直接返回
		c.JSON(consts.StatusOK, entity.HttpResult{Status: true, Code: "", Message: "mapBuf文件夹数据一致。"})
	} else {
		var firstValue int
		for i, id := range req {
			// 根据id获取对应的oss文件列表
			fileList, err := util.GetExactedMapFileById(id)
			// 过滤特定后缀的文件列表
			fileList = util.FilterBySuffixes(fileList, config.MapBufFiltersuffixes...)
			//fmt.Println("Filtered Strings:", fileList)

			if err != nil {
				c.String(consts.StatusBadRequest, err.Error())
				return
			}

			// 获取文件列表的总大小
			totalSize := calculateTotalFileSize(fileList)
			fmt.Println("Total Size:", totalSize)

			// 判断不同文件列表（mapBuf）中文件的总大小是否一致
			if i == 0 {
				firstValue = totalSize
			} else {
				if totalSize != firstValue {
					c.JSON(consts.StatusOK, entity.HttpResult{Status: false, Code: "", Message: "mapBuf文件夹数据不一致。"})
					return
				}
			}
		}

		c.JSON(consts.StatusOK, entity.HttpResult{Status: true, Code: "", Message: "mapBuf文件夹数据一致。"})
	}
}

// DownloadOSSFile 根据objectKey下载指定的oss文件
// @router /map/downloadossfile [GET]
func DownloadOSSFile(ctx context.Context, c *app.RequestContext) {
	objectKey := c.Query("objectKey")
	// 从OSS下载文件
	reader, err := config.OssBucket.GetObject(objectKey)

	if err != nil {
		c.JSON(http.StatusInternalServerError, map[string]string{"error": err.Error()})
		return
	}
	defer reader.Close()

	// 设置响应头
	c.Response.Header.Set("Content-Disposition", fmt.Sprintf(`attachment; filename="%s"`, filepath.Base(objectKey)))
	c.Response.Header.Set("Content-Type", "binary/octet-stream")

	// 将文件流式传输回客户端
	data, err := io.ReadAll(reader)
	if err != nil {
		panic(err)
	}
	if _, err := c.Write(data); err != nil {
		c.JSON(http.StatusInternalServerError, map[string]string{"error": err.Error()})
		return
	}
}

// DownloadMapZipFile 根据请求id从oss拉取并打包下载建图所需要的文件
// @router /map/downloadmapzipfile   [GET]
func DownloadMapZipFile(ctx context.Context, c *app.RequestContext) {
	id := c.Query("id")
	fmt.Println(id)

	// 根据id生成用于地图更新的压缩包
	filePath, tmpDir, err := generateMapZipById(id)
	if err != nil {
		c.JSON(http.StatusInternalServerError, map[string]string{"error": err.Error()})
	}
	fmt.Println("filePath", filePath)

	// 检查文件是否存在
	if _, err := os.Stat(filePath); os.IsNotExist(err) {
		c.JSON(http.StatusNotFound, map[string]string{"error": "File not found"})
		return
	}

	// 打开文件
	f, err := os.Open(filePath)
	if err != nil {
		c.JSON(http.StatusInternalServerError, map[string]string{"error": "Failed to open file"})
		return
	}
	defer f.Close()

	// 设置响应头
	c.Response.Header.Set("Content-Disposition", fmt.Sprintf(`attachment; filename="%s"`, filepath.Base(filePath)))
	c.Response.Header.Set("Content-Type", "binary/octet-stream")

	// 将文件流式传输回客户端
	data, err := io.ReadAll(f)
	if err != nil {
		panic(err)
	}
	if _, err := c.Write(data); err != nil {
		c.JSON(http.StatusInternalServerError, map[string]string{"error": err.Error()})
		return
	}
	defer os.RemoveAll(tmpDir)
}

// DownloadMapBagFile 根据请求id从oss拉取并下载解析后的map.bag文件
// @router /map/downloadmapbagfile   [GET]
func DownloadMapBagFile(ctx context.Context, c *app.RequestContext) {
	id := c.Query("id")
	fmt.Println("id: ", id)

	// 根据id获取对应的oss文件列表
	fileList, err := util.GetExactedMapFileById(id)
	// 过滤特定后缀的文件列表
	fileList = util.FilterBySuffixes(fileList, config.MapBagFiltersuffixes...)
	//fmt.Println("fileList", fileList)

	objectKey := fileList[0]
	// 从OSS下载文件
	reader, err := config.OssBucket.GetObject(objectKey)

	if err != nil {
		c.JSON(consts.StatusInternalServerError, entity.HttpResult{Status: false, Code: "", Message: "解析地图Bag下载失败。"})
		return
	}
	defer reader.Close()

	// 设置响应头
	c.Response.Header.Set("Content-Disposition", fmt.Sprintf(`attachment; filename="%s"`, filepath.Base(objectKey)))
	c.Response.Header.Set("Content-Type", "binary/octet-stream")

	// 将文件流式传输回客户端
	data, err := io.ReadAll(reader)

	if err != nil {
		panic(err)
	}
	if _, err := c.Write(data); err != nil {
		c.JSON(consts.StatusInternalServerError, entity.HttpResult{Status: false, Code: "", Message: "解析地图Bag下载失败。"})
		return
	}
	c.JSON(consts.StatusOK, entity.HttpResult{Status: true, Code: "", Message: "解析地图Bag下载成功。"})
}

// 根据id生成用于地图更新的压缩包
func generateMapZipById(id string) (file string, tmpDir string, err error) {
	// 根据id获取对应的oss文件列表
	allFileList, err := util.GetExactedMapFileById(id)

	// 过滤特定后缀的文件列表
	mapBufFileList := util.FilterBySuffixes(allFileList, config.MapBufFiltersuffixes...)
	//fmt.Println("Filtered Strings:", fileList)
	if err != nil {
		return
	}

	// 创建临时文件夹
	tmpDir, err = os.MkdirTemp("", "temp-download-*")

	fmt.Println("tmpDir:", tmpDir)
	if err != nil {
		fmt.Println("Error creating temporary directory:", err)
		return "", "", err
	}
	c_log.GlobalLogger.Info("创建下载-临时文件夹：", tmpDir)

	// 创建根文件夹（文件打包的根目录）
	baseDir := filepath.Join(tmpDir, "data")
	if err := os.Mkdir(baseDir, 0755); err != nil {
		fmt.Println("Error creating subdirectory:", err)
		return "", "", err
	}
	c_log.GlobalLogger.Info("创建文件打包根目录：", baseDir)

	// 根目录创建子文件夹 bag_folder
	bagFolderDir := filepath.Join(baseDir, "bag_folder")
	if err := os.Mkdir(bagFolderDir, 0755); err != nil {
		fmt.Println("Error creating subdirectory:", err)
		return "", "", err
	}
	c_log.GlobalLogger.Info("根目录下创建bag_folder子文件夹：", bagFolderDir)

	// 根目录创建子文件夹 origin_map_folder
	originMapFolderDir := filepath.Join(baseDir, "origin_map_folder")
	if err := os.Mkdir(originMapFolderDir, 0755); err != nil {
		fmt.Println("Error creating subdirectory:", err)
		return "", "", err
	}
	c_log.GlobalLogger.Info("根目录下创建origin_map_folder子文件夹：", originMapFolderDir)

	// 子文件夹 bag_folder 创建子文件夹 mapBuf
	mapBufDir := filepath.Join(bagFolderDir, "mapBuf")
	if err := os.Mkdir(mapBufDir, 0755); err != nil {
		fmt.Println("Error creating subdirectory:", err)
		return "", "", err
	}
	c_log.GlobalLogger.Info("bag_folder创建mapBuf子文件夹 ：", mapBufDir)

	// 从oss下载build_map.bag文件到 bag_folder 文件夹
	// 过滤特定后缀的文件列表
	buildMapBagFileList := util.FilterBySuffixes(allFileList, config.BuildMapBagFiltersuffixes...)
	buildMapBagFile := buildMapBagFileList[0]
	err = config.OssBucket.GetObjectToFile(buildMapBagFile, filepath.Join(bagFolderDir, filepath.Base(buildMapBagFile)))
	if err != nil {
		fmt.Println("Error downloading Bag file:", err)
		return "", "", err
	}
	c_log.GlobalLogger.Info("下载文件到bag_folder文件夹 - 成功")

	// 从oss下载文件到 mapBuf 文件夹
	for _, file := range mapBufFileList {
		err = config.OssBucket.GetObjectToFile(file, filepath.Join(mapBufDir, filepath.Base(file)))
		if err != nil {
			fmt.Println("Error downloading mapBuf file:", err)
			return "", "", err
		}
	}
	c_log.GlobalLogger.Info("下载文件到mapBuf子文件夹 - 成功")

	// 从oss下载bag文件到 origin_map_folder 文件夹
	// 过滤特定后缀的文件列表
	originMapFileList := util.FilterBySuffixes(allFileList, config.OriginMapFiltersuffixes...)
	for _, file := range originMapFileList {
		err = config.OssBucket.GetObjectToFile(file, filepath.Join(originMapFolderDir, filepath.Base(file)))
		if err != nil {
			fmt.Println("Error downloading origin_map file:", err)
			return "", "", err
		}
	}
	c_log.GlobalLogger.Info("下载文件到origin_map_folder文件夹 - 成功")

	// 创建压缩文件
	zipPath := filepath.Join(tmpDir, "mapFile-"+id+".zip")
	zipFile, err := os.Create(zipPath)
	if err != nil {
		fmt.Println("Error creating ZIP file:", err)
		return "", "", err
	}
	defer zipFile.Close()

	zipWriter := zip.NewWriter(zipFile)
	defer zipWriter.Close()

	// 压缩文件夹
	if err := util.AddDirToZip(baseDir, zipWriter); err != nil {
		fmt.Println("Error adding directory to ZIP:", err)
		return "", "", err
	}
	fmt.Println("ZIP file created successfully.")

	c_log.GlobalLogger.Info("创建压缩文件 - 成功")

	return zipPath, tmpDir, nil
}

// 计算oss中文件列表的总大小
func calculateTotalFileSize(fileList []string) int {
	var totalSize int
	for _, file := range fileList {
		size, err := util.GetOSSFileSize(config.OssBucket, file) // 获取oss中单个文件的大小
		if err != nil {
			return 0
		}
		totalSize += size
	}
	return totalSize
}
